import com.j256.ormlite.dao.Dao;
import com.j256.ormlite.dao.DaoManager;
import com.j256.ormlite.jdbc.JdbcConnectionSource;
import com.j256.ormlite.support.ConnectionSource;

import java.sql.SQLException;
import java.util.InputMismatchException;
import java.util.List;
import java.util.Scanner;

public class AccountApp {

    // we are using a MySQl database
    private final static String DATABASE_URL = "jdbc:mysql://localhost:3306/orm_lab?useUnicode=true";

    private Dao<Account, Integer> accountDao;

    public static void main(String[] args) throws Exception {
        // turn our static method into an instance of Main
        new AccountApp().doMain(args);
    }

    private void doMain(String[] args) throws Exception {
        ConnectionSource connectionSource = null;
        try {
            // create our data-source for the database
            connectionSource = new JdbcConnectionSource(DATABASE_URL, "root", "");
            // setup our  DAOs
            setupDao(connectionSource);
            // read, write and delete some data
            processData();

            System.out.println("\n\nIt seems to have worked\n\n");
        } finally {
            // destroy the data source which should close underlying connections
            if (connectionSource != null) {
                connectionSource.close();
            }
        }
    }

    /**
     * Read and write some example data.
     */
    private void processData() throws Exception {
        Scanner s = new Scanner(System.in);
        while (true) {
            printDivider();
            System.out.println("Type 'Create' to add a new user.");
            System.out.println("Type 'Read' to view the information for all accounts.");
            System.out.println("Type 'Update' to edit an existing account.");
            System.out.println("Type 'Delete' to remove a user.");

            System.out.println("Type anything else to quit");

            String input = s.nextLine();
            printDivider();

            if (input.equalsIgnoreCase("create")) {
                printDivider();
                System.out.print("Enter a name for the account: ");
                String name = s.nextLine();
                // create an instance of Account
                Account account = new Account(name);
                // persist the account object to the database
                accountDao.create(account);
                int id = account.getId();
                System.out.println(id);
                // assign a password
                System.out.print("Enter a password for the account: ");
                String password = s.nextLine();
                printDivider();
                account.setPassword(password);
                // update the database after changing the object
                accountDao.update(account);

            }
            else if (input.equalsIgnoreCase("read")) {
                printDivider();
                List<Account> accounts = accountDao.queryForAll();
                System.out.println();
                System.out.println("----------------------------------------------------");
                System.out.println(String.format("|%6s||%20s||%20s|", "id", "name", "password"));
                System.out.println("----------------------------------------------------");
                for (Account a : accounts) {
                    System.out.println(String.format("|%6d||%20s||%20s|", a.getId(), a.getName(), a.getPassword()));
                    System.out.println("----------------------------------------------------");
                }
                printDivider();
            }
            else if (input.equalsIgnoreCase("update")) {
                printDivider();
                System.out.print("Enter id for account to update: ");
                Account a;
                while (true) {
                    try {
                        int id = s.nextInt();
                        s.nextLine();
                        a = accountDao.queryForId(id);
                        break;
                    }
                    catch (InputMismatchException | SQLException e2) {
                        System.out.print("Invalid input. Please enter a valid id: ");
                    }
                }
                System.out.print("Enter field to update ('name' or 'password'): ");
                String updateField = s.nextLine();
                while (true) {
                    if (updateField.equalsIgnoreCase("name")) {
                        System.out.println("New name: ");
                        a.setName(s.nextLine());
                        break;
                    }
                    else if (updateField.equalsIgnoreCase("password")) {
                        System.out.println("New password: ");
                        a.setPassword(s.nextLine());
                        break;
                    }
                }
                accountDao.update(a);
                printDivider();

            }
            else if (input.equalsIgnoreCase("delete")) {
                System.out.print("Enter id for account to delete: ");
                while(true) {
                    try {
                        int id = s.nextInt();
                        s.nextLine();
                        // delete the account
                        accountDao.deleteById(id);
                        break;
                    } catch (InputMismatchException | SQLException e2) {
                        System.out.print("Invalid input. Please enter a valid id: ");
                    }
                }

            }
            else {
                break;
            }


        }
    }

    /**
     * Setup our  DAOs
     */
    private void setupDao(ConnectionSource connectionSource) throws Exception {

        accountDao = DaoManager.createDao(connectionSource, Account.class);

    }

    private void printDivider() {
        System.out.println();
        System.out.println("####################################################");
        System.out.println();
    }
}
